/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/

var sandboxRun = function (message, resourceLocator) {
    return {
        execute: function (callback) {
            console.info("SANDBOX - " + message.correlationId + ' - Executing command echo ' + message.payload);

            resourceLocator.sandboxRunner(message.payload,
                createFailure(message, resourceLocator, callback),
                createSuccess(message, resourceLocator, callback));
        },
        describe: function () {
            return message.correlationId + " - Run in Sandbox, device MAC: :" + message.payload.device.hw_address;
        }
    };
};

function createSuccess(message, resourceLocator, callback) {
    return function success(result) {
        console.info("SANDBOX On success");
        if (result  && result.variables) {
            var engine = resourceLocator.engineRequest;

            engine.sendWithPromise(
                '/device/' + message.payload.device.id + '/variable', 'PUT', result.variables
            ).then(function () {
                console.info("SANDBOX Variables sent back to engine");
            });
        }
        return callback(result);
    };
}

function createFailure(_, __, callback) {
    return function failure(result) {
        console.warn("SANDBOX Failure: %s", JSON.stringify(result));
        return callback(result);
    };
}

module.exports.command = sandboxRun;