/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Created by Iacopo Papalini <ipapalini@domotz.com> on 09/04/15.
 *
 */

var updateConfiguration = function (message, resourceLocator) {

    var payload = message.payload;

    var configurator = resourceLocator.configurator;
    var killer = resourceLocator.utils.proc.gracefulShutdown;
    var credentials = resourceLocator.configuration.credentials;
    var httpOutputCache = resourceLocator.httpOutCache;

    var reader = configurator.getConfiguration;
    var writer = configurator.writeConfiguration;
    var setConfigurationUrl = configurator.setConfigurationUrl;
    var removeOtherSettingsFiles = configurator.removeOtherSettingsFiles;

    return {
        describe: function () {
            return "Update Configuration - Version: " + payload.version;
        },
        execute: function (callback) {
            console.info('updateConfiguration - Require new config data to: ' +
                payload.location);

            setConfigurationUrl(payload.location);

            reader(credentials)
                .then(writer)
                .then(httpOutputCache.resetCache)
                .then(removeOtherSettingsFiles)
                .then(killer)
                .catch(function (err) {
                    console.warn('Error on updateConfiguration command');
                    console.error(err.stack);
                });

            callback();
        }
    };
};

module.exports.command = updateConfiguration;


