/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz UK LLP
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/
/**
 *
 * Created by Fausto Lelli <fausto@domotz.com>
 *
 */
const q = require('q');

var sequentialExecutor = function () {
    var executeSequentially = function (promises, myConsole) {
        myConsole.log("Execute sequentially # %d tasks " + promises.length);
        var counter = 0;
        var workers_done = 0;
        var acc = [];
        var result = q();
        var deferred = q.defer();
        promises.push(q);
        promises.forEach(function (worker) {
            result = result.then(function (taskResult) {
                counter++;
                if (taskResult) {
                    acc.push(taskResult);
                }
                myConsole.log("Starting new task " + counter + " ...");
                if ((promises.length - 1) === workers_done) {
                    myConsole.log("All tasks sequence terminated. returning ");
                    deferred.resolve(acc);
                }
                return worker();
            }).catch(function (err) {
                myConsole.log("Task received err " + err);
            }).finally(function () {
                workers_done++;
                myConsole.log("Task terminated. done (%d/%d)", workers_done, promises.length);
            });
        });
        return deferred.promise;
    };

    return {
        execute: executeSequentially
    };

};

var parallelExecutor = function () {

    var executeParallel =  function (promises, myConsole)  {
        return  q.all(promises);
    };

    return {
        execute: executeParallel
    };
};

module.exports.sequentialExecutor = sequentialExecutor;
module.exports.parallelExecutor = parallelExecutor;