/** This file is part of Domotz Agent.
 * Copyright (C) 2020  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 */

const LOG_EVENT = 'HANDLER - (snmpCheckDiscovery): ';

var counter = 1;
var isStartup = true;

var snmpCheckDiscovery = function (resourceLocator) {
    var snmpCheckDiscovery = resourceLocator.discovery.snmp_check_discovery;
    var logger = resourceLocator.log.decorateLogs(LOG_EVENT);
    var numberOfFastDiscoveryCycle = null;

    var nonStartupNumberOfFastDiscoveryCycle = resourceLocator.settingsManager.getMibDiscoveryInterval();
    var startupNumberOfFastDiscoveryCycle = resourceLocator.settingsManager.getMibDiscoveryStartupInterval();

    if(isStartup){
        numberOfFastDiscoveryCycle = startupNumberOfFastDiscoveryCycle;
    }
    else{
        numberOfFastDiscoveryCycle = nonStartupNumberOfFastDiscoveryCycle;
    }

    logger.debug('configured cycles of fast discoveries %s ', numberOfFastDiscoveryCycle);

    if (!numberOfFastDiscoveryCycle) {
        logger.info('avoiding snmp check discovery by settings value %s ', numberOfFastDiscoveryCycle);
        return;
    }

    logger.info('counter => %s / %s ', counter, numberOfFastDiscoveryCycle);

    if (counter === 0) {
        logger.debug('Performing SNMP check discovery ONESHOT');
        snmpCheckDiscovery.oneshotDiscovery();
        isStartup = false;
    }
    counter++;
    if (counter >= numberOfFastDiscoveryCycle) {
        counter = 0;
    }

};

module.exports.handler = snmpCheckDiscovery;
