/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/
/**
 * Created by Tommaso Latini <tommaso@domotz.com> on 18/09/15.
 */

const LOG_EVENT = 'HANDLER - (updateHttpListeningPort)';
var previousIp, previousPort;

var updateHttpListeningPort = function (resourceLocator) {

    try {
        var status = resourceLocator.status;

        var restricted = (status.get() === status.RESTRICTED || status.get() === status.UPDATING);
        var ipAddress = resourceLocator.discovery.host.getIP();


        var listeningPort = resourceLocator.webapp.get('port');

        if (restricted) {
            console.info(LOG_EVENT + ": Running in restricted or updating mode, don't signal the engine listening IP/port");
        } else if (!listeningPort) {
            console.info(LOG_EVENT + ": No listening port available, don't signal the engine");
        } else if (!ipAddress) {
            console.info(LOG_EVENT + ": No IP address available, don't signal the engine");
        } else if (ipAddress !== previousIp && listeningPort !== previousPort) {
            resourceLocator.engineRequest.sendWithPromise(
                '', 'PUT', {listen_on: ipAddress + ':' + listeningPort}).then(function () {
                console.info(LOG_EVENT + ": Updated listening IP [%s] and Port [%s] on " +
                    "Engine", ipAddress, listeningPort);
                previousIp = ipAddress;
                previousPort = listeningPort;
            });
        } else {
            console.debug(LOG_EVENT + ": No need to update engine. IP [%s] and " +
                "Port [%s] did not change.", ipAddress, listeningPort);
        }
    }
    catch(err){
        console.error(LOG_EVENT + ": error " + err);
    }

};

module.exports.handler = updateHttpListeningPort;
