/** This file is part of Domotz Agent.
  * Copyright (C) 2016  Domotz Ltd
  *
  * Domotz Agent is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * Domotz Agent is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
  **/
/**
 *
 * Created by Andrea Tondo <andrea@domotz.com> on 07/03/19.
 */

module.exports.post = function (resourceLocator, onException, onSuccess) {

    var configurator = resourceLocator.configurator;
    var httpOutputCache = resourceLocator.httpOutCache;
    var removeOtherSettingsFiles = configurator.removeOtherSettingsFiles;

    function getOnSuccessData(conf) {
        var agentId = conf.getConfiguredAgentId();
        if (agentId) {
            return {headers: {agentId: agentId}};
        }
        return {};
    }

    return function (req, res) {
        console.info('API - (%s): %s', req.method, req.url);

        var body = req.body;
        var configurationUrl = body.configurationUrl;
        var geoLocation = body.geoLocation;

        console.log('body');
        console.log(JSON.stringify(body));

        configurator.setGeographicalCoordinates(geoLocation);
        configurator.setConfigurationUrl(configurationUrl);

        configurator.sendGeographicalCoordinates()
            .then(configurator.getConfiguration)
            .then(configurator.writeConfiguration)
            .then(httpOutputCache.resetCache)
            .then(removeOtherSettingsFiles)
            .then(function () {
                onSuccess(res, getOnSuccessData(configurator));
            })
            .catch(function (err) {
                console.log(err);
                onException(res, err);
            });
    };
};