/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/

var create = function (express, status, configuration, resourceLocator) {

    var router = express.Router();

    var api_resources = [];

    api_resources.push(['configure', 'api/vendorAgent', ['post']]);
    api_resources.push(['status', 'api/status', ['get']]);
    api_resources.push(['log/flush', 'api/flush_log', ['get']]);
    api_resources.push(['location', 'api/location', ['get']]);
    api_resources.push(['activate', 'api/activate', ['post']]);
    api_resources.push(['restart', 'api/restart', ['post']]);
    api_resources.push(['random', 'api/random', ['get']]);
    api_resources.push(['agent', 'api/agent', ['post']]); // 404 is returned by handler if not in restricted mode

    if (status.isConfigured() || status.isPassive()) {
        api_resources.push(['agent/' + configuration.id, 'api/coupling', ['get']]);
        api_resources.push(['agent', 'api/agent', ['get']]);
        api_resources.push(['net-info', 'api/netinfo', ['get']]);
        api_resources.push(['mtr', 'api/mtr', ['get']]);
        api_resources.push(['ndt-server', 'api/ndtServer', ['get']]);
        api_resources.push(['network-speed?', 'api/networkSpeed', ['get']]);
        api_resources.push(['connectivity', 'api/connectivity', ['get']]);
        api_resources.push(['device', 'api/device', ['get']]);
        api_resources.push(['network-config', 'api/networkConfig', ['post', 'delete', 'get']]);
        api_resources.push(['vlan-config/:tag?', 'api/vlanConfig', ['get', 'post', 'delete']]);
        api_resources.push(['ping', 'api/ping', ['post']]);
        
        if (process.env.DPLATFORM === 'win') {
            api_resources.push(['process/clean', 'api/process_clean', ['get']]);
        }

        if (status.isPassive() ||
            process.env.ENABLE_HTTP_TEST_API ||
            resourceLocator.lodash.get(resourceLocator, 'configuration.settings.enable_http_test_api', false)) {
            api_resources.push(['event/trigger/:event_name', 'api/event', ['post']]);
            api_resources.push(['scp', 'api/scp', ['post']]);
            api_resources.push(['test/ssh_pass', 'api/test_ssh_pass', ['post']]);
            api_resources.push(['sshShellSequence', 'api/sshShellSequence', ['post']]);
            api_resources.push(['tftp', 'api/tftp', ['post']]);
            api_resources.push(['command/:command', 'api/command', ['post']]);
            api_resources.push(['sandbox/run', 'api/sandboxRunner', ['post']]);
        }
    }
    else if (status.isRestricted()) {
        api_resources.push(['agent/credentials', 'api/credentials', ['put']]);
    }

    console.info('WEBAPP - +------------- INTERNAL API ----------------------+');
    api_resources.forEach(function (route) {
        route[2].forEach(function (method) {
            var moduleName = '../request_handlers/' + route[1];
            var path = route[0];
            console.info('WEBAPP - Create route: ' + method.toUpperCase() + ' ' + path + '. File: ' + moduleName);
            var handler = require(moduleName)[method](resourceLocator, _onException, _onSuccess);
            router[method]('/api/v1/' + path, handler);
        });
    });

    console.info('WEBAPP - +-------------------------------------------------+');

    return router;
};

module.exports.create = create;

function _onSuccess(res, data, plain) {
    if (!data) {
        console.debug("Success, providing empty response");
        res.status(204);
    } else if (plain) {
        console.debug("Success, providing as plaintext");
        res.status(200).send(data);
    } else {
        if (data && data.headers) {
            console.debug("Success, providing as complex data");
            var headersParsed = JSON.parse(JSON.stringify(data.headers));
            headersParsed['content-encoding'] = undefined;
            for (var headerIndex in headersParsed) {
                res.header(headerIndex, headersParsed[headerIndex]);
            }
            var bodyParsed = data.body ? data.body : {};
            res.status(200).send(bodyParsed);
        }
        else {
            console.debug("Success, providing as json");
            res.status(200).send(JSON.stringify(data));
        }
    }
    res.end();
}

function _onException(res, err) {
    if (err === undefined) {
        err = {
            message: "Unknown error, see logs for further details",
            status: 500
        };
    }
    console.warn('Internal Error: ' + err.status + ' - ' + JSON.stringify(err.message || '[ No error message ]'));
    res.status(err.status || 500).send(JSON.stringify(err.message || {}));
    res.end();
}


