/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Iacopo Papalini <iacopo@domotz.com>
 */

const logger = function (level, maxSize) {
    var log = [];

    function nop() {
    }

    function create(prefix) {
        return function () {
            var msg = arguments[0];
            var format = require('util').format;
            for (var i = 1; i < arguments.length; i++) {
                msg = format(msg, arguments[i]);
            }
            log.push(prefix + ' ' + msg);
            if (log.length > maxSize) {
                log.shift();
            }
        };
    }

    var logger = {
        debug: create('D'),
        info: create('I'),
        warn: create('W'),
        error: create('E'),
        get: function () {
            return log;
        }
    };


    switch (level.toLowerCase()) {
        case 'info':
            logger.debug = nop;
            break;
        case 'warning':
            logger.debug = nop;
            logger.info = nop;
            break;
        case 'error':
            logger.debug = nop;
            logger.info = nop;
            logger.warn = nop;
            break;

    }
    logger.log = logger.debug;
    logger.warning = logger.warn;

    return logger;
};


module.exports.logger = logger;